# -*- coding: utf-8 -*-
#
# SPDX-FileCopyrightText: 2025 Florian Obersteiner <f.obersteiner@kit.edu>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import logging as log
from pathlib import Path

import dash_bootstrap_components as dbc
import pandas as pd
import xarray as xr
from dash import Dash

import src.data as loader
from src.callbacks import (
    register_info_updaters,
    register_map_plot,
    register_timeseries_plot,
)
from src.config import DotDict, get_latest_semantic_version_tag
from src.layout import create

# -----------------------------------------------------------------------------

appconfig: DotDict = DotDict.from_toml_file("./config/env.toml")
appconfig.VERSION = get_latest_semantic_version_tag() or appconfig.VERSION
log.basicConfig()
log.getLogger().setLevel(appconfig.LOGLEVEL)

varconfig: pd.DataFrame = pd.read_csv("./config/parms_units.csv", sep=";")
varconfig = varconfig.set_index("VNAME")

airportsconfig: pd.DataFrame = pd.read_csv("./config/CARIBIC2_Airports.csv", sep=";")
airportsconfig = airportsconfig.set_index("IATA")

# using separate variable declaration here to have a type annotation:
data: pd.DataFrame = pd.DataFrame()
trajdata: dict[int, xr.Dataset] = {}
ap_info: dict[int, str] = {}

# fmt: off
data, ap_info = (
    loader.ms_from_thredds(loader.get_ms_datasets(appconfig), appconfig, airportsconfig, True)
    if appconfig.USE_THREDDS
    else loader.ms_from_disk(
        Path("./testdata").glob(appconfig.FNAME_PREFIX + "*" + appconfig.FNAME_SUFFIX), 
        appconfig, 
        airportsconfig, 
        True)
)
trajdata = (
    loader.tr_from_thredds(appconfig)
    if appconfig.USE_THREDDS
    else loader.tr_from_disk(
        Path("./testdata").glob(appconfig.FNAME_PREFIX_TR + "*" + appconfig.FNAME_SUFFIX)
    )
)

# -----------------------------------------------------------------------------

app = Dash(
    appconfig.TITLE,
    requests_pathname_prefix=appconfig.REQUESTS_PATH_PREFIX,
    external_stylesheets=[dbc.themes.BOOTSTRAP, dbc.icons.FONT_AWESOME],
)
# fmt: on

app.title = appconfig.TITLE
app.css.config.serve_locally = True
app.scripts.config.serve_locally = True

app.layout = create(data, appconfig)
server = app.server  # for deployment via gunicorn / WSGI server


register_info_updaters(app, varconfig, ap_info)
register_map_plot(app, data, trajdata, varconfig)
register_timeseries_plot(app, data, varconfig)


if __name__ == "__main__":
    app.run(debug=appconfig.DEBUG)  # type: ignore
